@@ -446,6 +446,11 @@ class KeyboardEvent(InputEvent):
 
         return self.event_string in ["space", " "]
 
+    def isFromApplication(self, app):
+        """Return True if this is associated with the specified app."""
+
+        return self._app == app
+
     def isCharacterEchoable(self):
         """Returns True if the script will echo this event as part of
         character echo. We do this to not double-echo a given printable
@@ -708,7 +713,7 @@ class KeyboardEvent(InputEvent):
                     debug.println(debug.LEVEL_INFO, "Done with capslock", True)
                 except:
                     debug.println(debug.LEVEL_INFO, "Could not trigger capslock, " \
-                        "at-spi2-core >= 2.30 is needed for triggering capslock", True)
+                        "at-spi2-core >= 2.32 is needed for triggering capslock", True)
                     pass
             return lockit
         debug.println(debug.LEVEL_INFO, "Scheduling capslock", True)
