@@ -874,7 +874,9 @@ class Script(default.Script):
             return
 
         selectedChildren = self.utilities.selectedChildren(event.source)
-        if len(selectedChildren) == 1:
+        if len(selectedChildren) == 1 \
+           and self.utilities.containingComboBox(event.source) == \
+               self.utilities.containingComboBox(orca_state.locusOfFocus):
             orca.setLocusOfFocus(event, selectedChildren[0], True)
 
     def onTextSelectionChanged(self, event):
@@ -887,7 +889,7 @@ class Script(default.Script):
 
         if event.source != orca_state.locusOfFocus \
            and event.source.getState().contains(pyatspi.STATE_FOCUSED):
-            orca.setLocusOfFocus(event, event.source, True)
+            orca.setLocusOfFocus(event, event.source, False)
 
         super().onTextSelectionChanged(event)
 
